
#include <CAENTypes.h>

int32_t _dppDescrInited = 0;

int32_t _compareDPPDescr(c_DPPDescriptor_t dpp1, c_DPPDescriptor_t dpp2, int32_t checkDPPCode) {
    if (dpp1.DPPType != dpp2.DPPType)
        return c_type_ErrorCode_WrongDPPTypeDescr;
    if (dpp1.ENType != dpp2.ENType)
        return c_type_ErrorCode_WrongEnergyDescr;
    if (checkDPPCode > 0 && dpp1.DPPCode != dpp2.DPPCode)
        return c_type_ErrorCode_WrongDPPCodeDescr;
    if (dpp1.TSType != dpp2.TSType)
        return c_type_ErrorCode_WrongTimeStampDescr;
    return c_type_ErrorCode_Success;
}

UTILS_DLLAPI int32_t UTILS_API c_initDPPDescr() {
    int32_t ret = c_type_ErrorCode_Success;
    if(_dppDescrInited)
        goto QuitFunction;

    DPP_CI_Descr.DPPType = c_DPPType_CI;
    DPP_CI_Descr.ENType = TYPE_INT16;
    DPP_CI_Descr.TSType = TYPE_UINT32;
    DPP_CI_Descr.DPPCode = -1;

    DPP_PSD_Descr.DPPType = c_DPPType_PSD;
    DPP_PSD_Descr.TSType = TYPE_UINT32;
    DPP_PSD_Descr.ENType = TYPE_INT32;
    DPP_PSD_Descr.DPPCode = -1;

    DPP_PHA_Descr.DPPType = c_DPPType_PHA;
    DPP_PHA_Descr.TSType = TYPE_UINT64;
    DPP_PHA_Descr.ENType = TYPE_UINT16;
    DPP_PHA_Descr.DPPCode = -1;

    DPP_None_Descr.DPPType = c_DPPType_None;
    DPP_None_Descr.TSType = TYPE_INT8;
    DPP_None_Descr.ENType = TYPE_INT8;
    DPP_None_Descr.DPPCode = -1;

    _dppDescrInited = 1;

QuitFunction:
    return ret;
}

// TODO 
UTILS_DLLAPI int32_t UTILS_API c_isDPPDescriptorValid(c_DPPDescriptor_t DPPDescr) {
    int32_t ret = -1;
    c_DPPDescriptor_t other;

    if ((ret = c_initDPPDescr()) != c_type_ErrorCode_Success)
        goto QuitFunction;

    switch(DPPDescr.DPPType) {
    case c_DPPType_CI:
        other = DPP_CI_Descr;
        break;
    case c_DPPType_PHA:
        other = DPP_PHA_Descr;
        break;
    case c_DPPType_PSD:
        other = DPP_PSD_Descr;
        break;
    default:
        // No valid DPPType Specified
        return -2;
        break;
    }

    ret = _compareDPPDescr(DPPDescr, other, 0);
    // Allow greater timestamps of the same sign
    if ((other.TSType == TYPE_UINT32 && DPPDescr.TSType == TYPE_UINT64) || (other.TSType == TYPE_INT32 && DPPDescr.TSType == TYPE_INT64))
        ret = c_type_ErrorCode_Success;

QuitFunction:
    return ret;
}

UTILS_DLLAPI int32_t UTILS_API c_getSizeOfType(c_type_t t) {
    switch(t) {
    case TYPE_INT8: {
        return sizeof(int8_t);
                    }
    case TYPE_UINT8: {
        return sizeof(uint8_t);
                     }
    case TYPE_INT16: {
        return sizeof(int16_t);
                     }
    case TYPE_UINT16: {
        return sizeof(uint16_t);
                      }
    case TYPE_INT32: {
        return sizeof(int32_t);
                     }
    case TYPE_UINT32: {
        return sizeof(uint32_t);
                      }
    case TYPE_INT64: {
        return sizeof(int64_t);
                     }
    case TYPE_UINT64: {
        return sizeof(uint64_t);
                      }
    case TYPE_LONG: {
        return sizeof(int32_t);
                    }
    case TYPE_DOUBLE: {
        return sizeof(double);
                      }
    case TYPE_CHAR: {
        return sizeof(char);
                    }
    case TYPE_SHORT: {
        return sizeof(int16_t);
                     }

    case TYPE_RESCODE:
    case TYPE_STRING:
    case TYPE_MEMORY:
    case TYPE_NONE:
    default: {
        return -1;
             }
    }
}

// string right trim
UTILS_DLLAPI size_t UTILS_API c_srtrim(char *str, const char *remove) {
    size_t l = strlen(str);
    while (strchr(remove, str[l - 1]) != NULL && l > 0) {
        str[l - 1] = '\0';
        l = strlen(str);
    }
    return strlen(str);
}

// string left trim
UTILS_DLLAPI size_t UTILS_API c_sltrim(char *str, const char *remove) {
    char *ptr = str;
    size_t i, l;
    while (strchr(remove, ptr[0]) != NULL && strlen(ptr) > 0)
        ptr++;
    l = strlen(ptr);
    for (i = 0; i <= l; i++)
        str[i] = ptr[i];
    return strlen(str);
}

// string trim
UTILS_DLLAPI size_t UTILS_API c_strim(char *str, const char *remove) {
    if (c_srtrim(str, remove) == 0)
        return 0;
    if (c_sltrim(str, remove) == 0)
        return 0;
    return strlen(str);
}

// override malloc/calloc/free functions to give
// external application control on memory
UTILS_DLLAPI void* UTILS_API c_malloc(size_t size) {
    return malloc(size);
}
UTILS_DLLAPI void* UTILS_API c_calloc(size_t count, size_t size) {
    return calloc(count, size);
}
UTILS_DLLAPI void UTILS_API c_free(void* memory) {
    free(memory);
}

#ifdef _DEF_UTILS_LIB_
#ifdef WIN32
/**********************************************************************\
                                DllMain
\**********************************************************************/
BOOL WINAPI DllMain(HANDLE hModule, DWORD ul_reason_for_call, LPVOID lpReserved) {
	UNREFERENCED_PARAMETER(hModule);
	UNREFERENCED_PARAMETER(lpReserved);
	switch (ul_reason_for_call) {
	case DLL_PROCESS_ATTACH:
	case DLL_PROCESS_DETACH:
	case DLL_THREAD_ATTACH:
	case DLL_THREAD_DETACH:
		break;
	}
	return TRUE;
}
#else
/**********************************************************************\
                        CAEN_DGTZ_init
\**********************************************************************/
void __attribute__ ((constructor)) CAEN_DGTZ_init(void) {
}
#endif
#endif