#ifdef DPP_TF_CORE

#include <stdlib.h>

#include "globs.h"
#include "Configurator.h"
#include "Digitizer.h"
#include "Analyzer.h"
#include "Controller.h"
#include "Plotter.h"
#include "Dumper.h"


int main(int argc, char* argv[])  {
	int error = 0;
	int loopctl = 0;
	Plotter*      plo = newPlotter();
	Dumper*		  dpr = newDumper();
	Digitizer*    dig = newDigitizer();
	Analyzer*     ana = newAnalyzer(dig, plo, dpr);
	Configurator* con = newConfigurator(argc>1 ? argv[1] : NULL, dig, ana, plo, dpr);
	Controller*	  ctl = newController(con, dig, ana, plo);

	//loopctl = doControlSequence(ctl);	
	if (error = parseConfiguration(con)) quitProgram(error);

	while(loopctl != CTL_QUIT && !error) { /* loop until quit or error from internal loop */
		
		if (error = initDigitizer(dig,0))	 break;
		if (error = initAnalyzer(ana))       break;

		do { /* loop until quit, restart or error */
			if (loopctl = doControl(ctl))   break; /* loop control is NOCMD, QUIT, or RESTART */
			if (error = doAcquisition(dig)) break;
			if (error = doAnalysis(ana))    break;
		} while(1);

		deinitAnalyzer(ana);
		deinitDigitizer(dig,0);

	};

	deleteController(ctl);
	deleteConfigurator(con);
	deleteAnalyzer(ana);
	deleteDigitizer(dig);
	deletePlotter(plo);

	quitProgram(error);
	return 0; /* control won't really get here. it's just to get rid of warnings */
}


#endif