

puts "loading sys.V1720.tcl"

variable registerlist

lappend registerlist [list NetVar.V1720_NumSamples  "-n V1720_NumSamples -t int"]
lappend registerlist [list NetVar.V1720_VMERetries  "-n V1720_VMERetries -t int"]
lappend registerlist [list NetVar.V1720_BERREnable  "-n V1720_BERREnable -t int"]
lappend registerlist [list NetVar.V1720_VMEBufSize  "-n V1720_VMEBufSize -t int"]

variable V1720_Base 160

proc Set_V1720_Options {} {
    variable Options
    variable V1720_Base

puts "Set_V1720_Options"

    load_Options

   if {![info exists Options(V1720_MaxEvtsPerBuf)]}         {set Options(V1720_MaxEvtsPerBuf) 1}
   if {![info exists Options(V1720_MaxBLT)]}                {set Options(V1720_MaxBLT) 1}
   if {![info exists Options(V1720_MemBufSize)]}            {set Options(V1720_MemBufSize) 1024}
   if {![info exists Options(V1720_VMEBufSize)]}            {set Options(V1720_VMEBufSize) 16384}

   if {![info exists Options(V1720_Hist_Trace.Enable)]}    {set Options(V1720_Hist_Trace.Enable) 1}
   if {![info exists Options(V1720_Hist_Trace.channels)]}  {set Options(V1720_Hist_Trace.channels) 512}
   if {![info exists Options(V1720_Hist_Trace.offset)]}    {set Options(V1720_Hist_Trace.offset) 0}
   if {![info exists Options(V1720_Hist_Trace.shift)]}     {set Options(V1720_Hist_Trace.shift) 0}

   if {![info exists Options(V1720_GroupBase)]}             {set Options(V1720_GroupBase) $V1720_Base}
   set V1720_Base  $Options(V1720_GroupBase)

   set Options(V1720_DPP) 0

    update_Options
}

proc V1720_Configure {} {
    variable Options

puts "V1720_Configure"

    set VME_BUFFER_SIZE [ReadNetVar NetVar.V1720_VMEBufSize]
    set Options(V1720_VMEBufSize) $VME_BUFFER_SIZE

      update_Options

#    WriteNetVar NetVar.V1720_Trig $Options(V1720_Trig)
#    WriteNetVar NetVar.V1720_DVP  $Options(V1720_DVP)
#    WriteNetVar NetVar.V1720_Diag  $Options(V1720_Diag)

           switch $Options(V1720_MemBufSize) {
              65536 {set BUFF_ORG 2; set BUFF_NUM 4}
              32768 {set BUFF_ORG 3; set BUFF_NUM 8}
              16384 {set BUFF_ORG 4; set BUFF_NUM 16}
              8192  {set BUFF_ORG 5; set BUFF_NUM 32}
              4096  {set BUFF_ORG 6; set BUFF_NUM 64}
              2048  {set BUFF_ORG 7; set BUFF_NUM 128}
              1024  {set BUFF_ORG 8; set BUFF_NUM 256}
              512   {set BUFF_ORG 9; set BUFF_NUM 512}
              256   {set BUFF_ORG 10; set BUFF_NUM 1024}
              default {set BUFF_ORG 10; set BUFF_NUM 1024}
           }

      set z [catch {ReadVMERegisterAttr v1720#1.VMEStatus} m]
      if {$z==0} then {

           WriteVMERegister v1720#1.VMEControl 16   ;# force BERR option

           set SIZE [ReadVMERegister v1720#1.CustomSize]
           if {$SIZE == 0} {set SIZE 32}
           WriteNetVar NetVar.V1720_NumSamples $SIZE

           set NUM [expr (($Options(V1720_MemBufSize) - 2) * 2) / $SIZE]
           if {$NUM > $Options(V1720_MaxEvtsPerBuf)} {set NUM $Options(V1720_MaxEvtsPerBuf)}

           set BERR [ReadVMERegister v1720#1.VMEControl]  ;#  enable BERR to finish a VME block transfer
           WriteNetVar NetVar.V1720_BERREnable [expr ($BERR >> 4) & 1]

puts "V1720 Output buffers $BUFF_NUM  @ $Options(V1720_MemBufSize) 32 bit words"
puts "Waveform [expr $SIZE * 2] samples"
puts "8 channels; $NUM events/buffer"

           set osize [expr (($SIZE + 2) * $NUM) + 2]
puts "V1720 buffer size used $osize words out of max $Options(V1720_MemBufSize)"

           if {$osize > $Options(V1720_MemBufSize)} {
              puts "Number of samples or Events per buffer is too big\n\
                 $osize > $Options(V1720_MemBufSize) - this is not permitted\n\
                 correct options and try again"
              return
           }

           set BLT [expr $VME_BUFFER_SIZE / (($osize * 8) + 4)]
           if {$BLT > $Options(V1720_MaxBLT)} {set BLT $Options(V1720_MaxBLT)}

puts "BLT Event Number = $BLT"
           set isize [expr (($osize * 8) + 4) * $BLT]
puts "Data Acq Input buffer size used $isize words out of max $VME_BUFFER_SIZE"

puts "V1720_Configure $BERR $SIZE $BUFF_ORG $NUM"

           WriteVMERegisters v1720#*.VMEControl $BERR     ;# set all modules to same setting
           WriteVMERegisters v1720#*.CustomSize $SIZE     ;# set all modules to same setting
           WriteVMERegisters v1720#*.BufferOrg $BUFF_ORG
           WriteVMERegisters v1720#\[?,??\].EvtsPerBuf $NUM
           WriteVMERegisters v1720#*.BLTEventNum $BLT     ;# set all modules to same setting
      }

puts "V1720_Configure finished"

}


proc V1720_DAQ-go {} {

   set maxmodule 0
   for {set i 1} {$i <= 16} {incr i} {
      set z [catch {ReadVMERegisterAttr v1720#$i.VMEStatus} m]
      if {$z!=0} then {break}
      incr maxmodule
   }

   for {set i 1} {$i <= $maxmodule} {incr i} {

      set z [catch {ReadVMERegister v1720#$i.AcqControl} m]
      if {$z == 0} {WriteVMERegister v1720#$i.AcqControl [expr ($m & 0xfb) + 4]}

   }
}

proc V1720_DAQ-halt {} {

   set maxmodule 0
   for {set i 1} {$i <= 16} {incr i} {
      set z [catch {ReadVMERegisterAttr v1720#$i.VMEStatus} m]
      if {$z!=0} then {break}
      incr maxmodule
   }

   for {set i 1} {$i <= $maxmodule} {incr i} {

      set z [catch {ReadVMERegister v1720#$i.AcqControl} m]
      if {$z == 0} {WriteVMERegister v1720#$i.AcqControl [expr ($m & 0xfb)]}

   }
}

proc V1720_DAQ-setup {} {

     V1720_Configure
}

proc V1720_DAQ-flush {} {}

proc V1720_DAQ-reset {} {}

proc define_V1720_config {} {
    variable Options
    variable V1720_Base

#   set maxmodule [inform vmecontrol inquire v1720count]
#   if {[string length $maxmodule] > 2} {set maxmodule 0}

puts "define_V1720_config"

   set maxmodule 0
   for {set i 1} {$i <= 16} {incr i} {
      set z [catch {ReadVMERegisterAttr v1720#$i.VMEStatus} m]
      if {$z!=0} then {break}
      incr maxmodule
   }

   if {$maxmodule > 0} then {DefineConfig "17200 0 [format "0x%x" $V1720_Base]"}

   for {set i 1} {$i <= $maxmodule} {incr i} {

      set z [catch {ReadVMERegisterAttr v1720#$i.Output_Buffer} m]
      if {$z!=0} then {
          puts "Attempting to locate register v1720#$i.Output_Buffer"
          error $m
      }

      DefineConfig "17200 $i [string range [first $m] 2 end]"
   }
}

proc create_V1720_spectra {} {
    variable V1720_Base
    variable Options 

puts "create_V1720_spectra"

   set maxmodule 0
   for {set i 1} {$i <= 16} {incr i} {
      set z [catch {ReadVMERegisterAttr v1720#$i.VMEStatus} m]
      if {$z!=0} then {break}
      incr maxmodule
   }

   set UseGroup [ReadNetVar NetVar.UseGroup]

   for {set i 1} {$i <= $maxmodule} {incr i} {

     if {$Options(V1720_Hist_Trace.Enable) == 1} {

        for {set j 0} {$j < 8} {incr j} {   

#    histograms for the data trace

           set spectrumname v1720#$i.Ch[set j]
           set z [catch {CreateSpectrum1D $spectrumname $Options(V1720_Hist_Trace.channels)} m]

#    Allow "Spectrum already exists" so that further spectra can be created
           if {$z!=0 && $m==0x30004} then {set z 0}
           if {$z!=0} then {
              puts "Attempting to create spectrum $spectrumname"
              error $m
           } else {
              set group  [expr (($i-1) * 8) + $j + 256  + $V1720_Base] 
              map_spectrum  "$spectrumname $group 0 $Options(V1720_Hist_Trace.offset) $Options(V1720_Hist_Trace.shift)"
           }
        }
     }
   }
}

